
# (c) COPYRIGHT 1996 URI/MIT
# Please read the full copyright statement in the file COPYRIGH.  
#
# Authors:
#      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

# The progress indicator popup. The progress indicator shows the user how
# much of the data request has been transfered from the server to the client
# and how much remains. It is continually updated during the transmission
# process and includes information on URL lookup an resolution.
#
# The progress indicator is a separate program which the DODS core software
# communicates with during data tranmissions. Users can replace this program
# with one of their own creation or can remove it altogether (thus disabling
# the progress indicator).
# 
# To run this code, start wish and then source this file. Feed commands to it
# via the `progress' procedure. wish must be started in its interactive mode
# so that commands canbe entered from stdin.
#
# jhrg 5/29/96

# $Log: dods_gui.tcl,v $
# Revision 1.9  2000/04/07 00:19:05  jimg
# Merged Brent's changes for the progress gui - he added a cancel button.
# Also repaired the last of the #ifdef Gui bugs so that we can build Gui
# and non-gui versions of the library that use one set of header files.
#
# Revision 1.8.4.2  2000/04/04 05:21:39  brent
# put a Cancel button the Tcl/Tk GUI
#
# Revision 1.8.4.1  2000/04/04 05:00:24  brent
# put a Cancel button the Tcl/Tk GUI
#
# Revision 1.4  1997/06/09 21:01:04  jimg
# Added tests for presence of $env(DODS_ROOT). In its absence use values of
# DODS_ROOT set at the time of the build. This means that many users of the
# binary distribution will have to set DODS_ROOT in the shell/rc script to get
# the GUI to work.
#
# Revision 1.3  1997/04/11 19:31:25  jimg
# Changed the two `source' lines so that they now use th DODS_ROOT environment
# variable.
#
# Revision 1.2  1996/11/18 21:51:22  jimg
# Added newline at end of file - configure was leaving the last line out of
# the generated file.
#
# Revision 1.1  1996/11/15 23:23:43  jimg
# Created from dods_gui.tcl - now configure sets the DODS_ROOT pathname so that
# the source commands work regardless of the directory from which this script
# is run.
#
# Revision 1.5  1996/06/22 00:10:07  jimg
# Changed error window title.
# Moved progress code from dods_gui.tcl - the code is now used to
# initialize the GUI which is spread among several files.
#
# Revision 1.4  1996/06/19 00:01:59  jimg
# Made the text widget fixed in size.
# Made the bar fill the canvas widget size (a real hack there).
#
# Revision 1.3  1996/06/14 22:47:50  jimg
# Changed the value of PERCENT in progress_bar that causes the `bouncing rect'
# from 0 to -1 since wwwlib uses a value of zero the first time it reports the
# amount of a document received.
# Removed the exit case in the progress procedure; to get the GUI to exit just
# send the program the string `exit\r'. This is independent of the progress
# indicator per se.
#
# Revision 1.2  1996/06/13 22:02:48  jimg
# Added `puts OK' to the end of the program so that expect can tell when it has
# been loaded.
#
# Revision 1.1  1996/06/12 20:24:57  jimg
# Created.
#

wm withdraw .                 ;# Hide wish's toplevel window

set font -*-courier-medium-r-normal--*-100-*-*-*-*-*-*
# Global variables used by the progress indictor.

set err_geometry +200+200
set pass_geometry +200+200
set progress_geometry +30+0
set progress_cancel_button "OK" 

# Global variables used by the progress indictor.

# The previous position of the progress bar (used when PERCENT is -1; see the
# progress_bar procedure. This value forces the initial box to be displayed
# flush with the left-hand side of the widget.
set progress_bar_pos -1

# The id of the last rectangle displayed in the progress bar; used to erase.
set progress_bar_rect 0
set sign +

proc progress_create {{text ""}} {
    global progress_geometry
    global font

    if {![winfo exists .progress]} {
        toplevel .progress
        wm withdraw .progress           ;# Hide the new toplevel window

        wm geometry .progress $progress_geometry
        wm title .progress "dods transmission"

        frame .progress.f -width 60m -height 30m

        label .progress.f.icon -bitmap info
        label .progress.f.msg -font $font -text $text -borderwidth 2 \
		-width 22 -anchor n

       canvas .progress.f.bar -relief sunken -width 56m -height 3m \
                -borderwidth 2

	button .progress.f.cancel -text Cancel \
		-command {set progress_cancel_button "CANCEL" }

        pack .progress.f -padx 6 -pady 6
        pack .progress.f.icon -side left -padx 20 -pady 20 -anchor nw
        pack .progress.f.msg -side top -padx 6 -pady 6 -fill y -expand true
	pack .progress.f.cancel -side bottom -pady 6 -anchor s
	pack .progress.f.bar -side bottom -padx 4 -pady 4 -anchor e

    }
}

# Replace the current text in the progress indicator with NEW_TEXT.

proc text {{new_text ""}} {
    .progress.f.msg configure -text $new_text
}

# Update progress bar to show percent transferred. If PERCENT is -1, then show
# a `bouncing rectangle' a la Netscape. If PERCENT is < -1 simply erase the
# bar and return.

proc bar {{percent -2}} {
    global progress_bar_pos
    global progress_bar_rect
    global sign
    global progress_cancel_button
    global usr_cancel

    # if the user has hit the CANCEL button on main frame, it shows up here
    if {$progress_cancel_button == "CANCEL"} {
 	set usr_cancel 1    ;# so the calling C process knows the user quit
        return
    } else {
	set usr_cancel 0
    }

    # Erase old bar.
    if {$progress_bar_rect != 0} {
        .progress.f.bar delete $progress_bar_rect
    }

    if {$percent < -1} {
        return
    }

    # Use the pixel sizes (from the widget) rather than the millimeter sizes
    # from the globals.
    set bar_height [.progress.f.bar cget -height]
    set bar_width [.progress.f.bar cget -width]

    # Bouncing rectangle?
    if {$percent == -1} {

        # Calculate NEW_POS and possibly flip SIGN
        # NB: progress_bar_pos is the left side of the little rectabgle,
        # and NEW_POS is the right side. The Rectangle is BOX_WIDTH pixels
        # long.
        set box_width [expr $bar_width / 5]
        set incr_size [expr $box_width / 2]
        if {$progress_bar_pos == -1} {
            set new_pos 0
        } else {
            set new_pos [expr $progress_bar_pos $sign $incr_size]
        }

        if {[expr $new_pos + $box_width] > $bar_width} {
            set sign -
            set new_pos [expr $bar_width - $box_width]
        } elseif {$new_pos < 0} {
            set sign +
            set new_pos $box_width
        }
       # Add a fudge factor to make the bar fill the canvas widget. (Why?)
        #set new_pos [expr $new_pos $sign 4]
        set bar_height [expr $bar_height + 4]

        # Now draw the rectangle
        set progress_bar_rect [.progress.f.bar create \
                rectangle [expr $new_pos - 4] 0 \
                          [expr $new_pos + $box_width + 4] $bar_height \
                -fill Blue ]

        # Update PROGRESS_BAR_POS for next time.
        set progress_bar_pos $new_pos
    } else {
        set progress_bar_pos [expr $bar_width * ($percent / 100.0)]

        # Add fudge factor...
        set progress_bar_pos [expr $progress_bar_pos + 4]
        set bar_height [expr $bar_height + 4]

        set progress_bar_rect [.progress.f.bar create rectangle 0 0 \
                $progress_bar_pos $bar_height -fill Blue -width 0]
    }
}

# Show the progress indicator window.

proc popup {{text ""}} {
    if {![winfo exists .progress]} {
        progress_create $text
    } elseif {[string length $text] != 0} {
    	text $text
    }
    wm deiconify .progress
}

# Hide the progress indicator window.

proc popdown {} {
    if {[winfo exists .progress]} {
        wm withdraw .progress
    }
}

# Prompt for a username and password. Return the two items in a list.

proc password {} {
    global font
    global pass_geometry
    global pass_button		;# Used in -command; must be global 

    global progress_pwd		# these have to be global to work
    global progress_usr
 
    if {![winfo exists .pass]} {
	toplevel .pass
	wm withdraw .pass

	wm geometry .pass $pass_geometry
	wm title .pass "Password"

	frame .pass.f -relief groove -borderwidth 1
	label .pass.f.icon -bitmap questhead
	frame .pass.f.lf
	label .pass.f.lf.user -text "Username:"
	label .pass.f.lf.pass -text "Password:"
	frame .pass.f.ef
	entry .pass.f.ef.user -font $font
	entry .pass.f.ef.pass -font $font -show "*"

	frame .pass.bf -relief groove -borderwidth 1
	frame .pass.bf.def -relief sunken -borderwidth 1
	button .pass.bf.def.ok -text "Ok" -command { set pass_button "OK" }
	button .pass.bf.cancel -text "Cancel" \
		-command {set pass_button "CANCEL" }

	pack .pass.f -side top -ipadx 10 -expand true -fill both 
	pack .pass.f.icon -side left -ipadx 20 -ipady 20 -anchor n
	pack .pass.f.lf -side left -anchor n -pady 10 
	pack .pass.f.lf.user -side top -anchor e -padx 2 -pady 2 -fill x \
		-expand true
	pack .pass.f.lf.pass -side top -anchor e -padx 2 -pady 2 -fill x \
		-expand true
	pack .pass.f.ef -side left -anchor n -pady 10
	pack .pass.f.ef.user -side top -anchor w -padx 2 -pady 2 -fill x \
		-expand true
	pack .pass.f.ef.pass -side top -anchor w -padx 2 -pady 2 -fill x \
		-expand true
	
	pack .pass.bf -side top -fill both -expand true -anchor s
	pack .pass.bf.def -side left -padx 5 -pady 5 -expand true
	pack .pass.bf.def.ok -side top -padx 5 -pady 5
	pack .pass.bf.cancel -side left -padx 5 -pady 5 -expand true

	bind .pass <Return> { set pass_button "OK" }
    } else {
	# If already built, erase the prvious values.
	.pass.f.ef.user delete 0 end
	.pass.f.ef.pass delete 0 end
    }

    focus .pass.f.ef.user 	;# Move to the first window

    wm deiconify .pass
    tkwait visibility .pass
    grab set .pass

    tkwait variable pass_button
    wm withdraw .pass
    grab release .pass

    puts $pass_button
    if {$pass_button == "OK"} {
	set progress_usr [.pass.f.ef.user get]
	set progress_pwd [.pass.f.ef.pass get]

    } elseif {$pass_button == "Cancel"} {
       set progress_usr ""         ;# set the values to a null char
       set progress_pwd ""
    }
}

proc simple_error {msg} {
    tk_dialog .tmp "Error" $msg error 0 "Ok"
}

# The interface to the progress indicator popup. Commands are fed to the
# popup using this procedure. Each takes the form `progress <command> ?arg?'
# where ?arg? is optional.

proc progress args {
    switch -regexp $args {
        "^popup*" {
            progress_popup [lindex $args 1]
            puts "OK"
        }
        "^popdown*$" {
            global progress_bar_pos
            set progress_bar_pos -1
            progress_bar -2
            progress_popdown
            puts "OK"
        }
        "^text*" {
            progress_text [lindex $args 1]
            puts "OK"
        }
        "^bar*" {
            progress_bar [lindex $args 1]
            puts "OK"
        }
        default {
            puts "Command unknown: $args"
        }
    }
    update
}

